#!/usr/bin/perl -w

use CGI qw/:standard *table *Tr *td *div/;
# http://www.geocities.co.jp/SiliconValley-Sunnyvale/6128/perl/cgipm.html
# http://www.tohoho-web.com/wwwperl1.htm

sub ln { "\n"; }

sub image {
    my ($e) = @_;
    "<img src=\"$e.png\" alt=\"$e\" align=\"middle\">";
}

$NATURAL = '<img src="natural.png">';  # without align

$dim = param('dim');
unless ($dim) {
    $dim = 3;
    param('dim', $dim);
}

print
    header(-expires=>'+1m'), # cache expires in 1min
    start_html(-title=> 'Separable Convex (M)'),
    a({href=>"./"},'DCP HOME'), ln,
    h1('Separable Convex Minimization (M<img src="natural.png" width=14> version)'), ln,
    '<BLOCKQUOTE>', ln;


sub set_param {
    my ($index, $min, $max) = @_;
    my ($e) = (param($index));
    if ( length($e)>0 and $e =~ m/([-+]?\d+)/
	 and $min <= int($1) and int($1) <= $max ) {
	$e = int($1);
    } else {
	$e = $min + int(rand($max-$min+1));
    }
    param($index, $e);
}

sub init_param {
    my ($i);

    for ($i=0; $i<$dim*2-1; $i++) {
	set_param("a:$i",   1, 50);
	set_param("b:$i", -99, 99);
    }
    for ($i=0; $i<$dim; $i++) {
	set_param("x:$i", -99, 99);
    }
}

sub print_html_table {
    init_param();
    print image('objfunc-separableM');
    print p, 'The conditions 0 < a', Sub(em('Y')) , '<100 and -100< b',
    Sub(em('Y')), '< 100 should be satisfied. ',
    'In the 4-dimentional case, ',
    image('calT'), 
    ' = {{1},{2},{3},{4}, {1,2}, {3,4}, {1,2,3,4}}.', ln;

    print <<EOF;

<p> This f(x) function is M$NATURAL-convex [1,2].
This web application minimizes f(x) using
<a href="http://www.misojiro.t.u-tokyo.ac.jp/~tutimura/odicon/">ODICON</a>.
EOF

    print start_form, hidden('dim'), start_div({-align=>'center'}), ln;

    @index = (1..9);
    print p, 'dimension = ', popup_menu(-name=>'newdim',-values=>[@index],-default=>$dim);
    print submit('Change'), br, ln;

    print start_table({-border=>'1'}), ln;
    my @td = 1..$dim;
    foreach $i (@td) { $i =~ s/(.*)/\{$1\}/; }
    print Tr({-align=>'center'}, td([@td])), ln;
    my $total = 0;
    for ($i=1; $i/2<$dim; $i*=2) {
	print start_Tr({-align=>'center'});
	for ($j=0; $j+$i/2<$dim; $j+=$i) {
	    my $max = $j+$i;  $max = $dim if $max > $dim;
	    my $x = 'x({' . join(',',($j+1)..$max) . '})';
	    print start_td({-colspan=>$i});
	    print textfield(-name=>"a:$total", -size=>3, -default=>param("a:$total")), $x, font({-size=>'-1'}, sup('2')), '+';
	    print textfield(-name=>"b:$total", -size=>3, -default=>param("b:$total")), $x;
	    print end_td, ln;
	    $total++;
	}
	print end_Tr, ln;
    }
    print end_table, ln;

    print br, start_table({-border=>'1'}), ln;
    print Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'}, [ '', 1 .. $dim ])]);
    foreach $e ('x') {
	my @td = ($e);
	for ($j=0; $j<$dim; $j++) {
	    push @td, textfield(-name=>"$e:$j", -size=>3, -default=>param("$e:$j"));
	}
	print Tr({-align=>'right'}, [td([@td])]), ln;
    }
    print end_table, ln;

    print p, "(Note: An inappropriate value will be replaced with a random number.)\n";
    print p, submit('Minimize'), reset('Reset'), ln;
    print end_div, end_form, ln;

    print <<EOF;
<hr width="70%">

<p> [1] K. Murota, "Discrete Convex Analysis---An Introduction (in Japanese),"
Kyoritsu Publishing Company, Tokyo, (2001).

<p> [2] K. Murota,
"Discrete Convex Analysis," SIAM, (2003). 
EOF
}

sub change_dim {
    $dim = param('newdim');
    param('dim', $dim);
}

sub call_executable {
    print '<pre>', ln;
    open(OUT, "|./odicon D $dim M") || die('Can not call ./odicon');
    #open(OUT, '|cat') || die('Can not call ./odicon');
    foreach $e ('a', 'b') {
	for ($j=0; $j<$dim*2-1; $j++) {
	    print OUT param("$e:$j"), ' ';
	}
	print OUT ln;
    }
    for ($j=0; $j<$dim; $j++) {
	print OUT param("x:$j"), ' ';
    }
    print OUT ln;
    close OUT;
    print '</pre>', ln;
}


if (param('Minimize')) {
    print_html_table();
    call_executable();
} elsif (param('Change')) {
    change_dim();
    print_html_table();
} else {
    print_html_table();
}

print '</BLOCKQUOTE>', end_html, ln;
